//*****************************************************************************
// Arquivo my_ints.c - tratamento das interrupções no PIC by Arne
// Desenvolvido por Márcio José Soares
//*****************************************************************************

//*****************************************************************************
// O Copyright deste programa está reservado para TESC Sistemas de Controle LTDA
// e seu conteúdo está protegido pela Lei de Direitos Autorais LEI Nº 9.610, de
// 19 de Fevereiro de 1998. É estritamente proibida a reprodução total ou
// parcial dos conteúdos aqui apresentados sem prévia autorização, por escrito,
// do autor.
//*****************************************************************************

//*****************************************************************************
// Funções do módulo
//*****************************************************************************

//*****************************************************************************
// Função para tratar o vetor de interrupções no PIC
//
//  Este é o tratamento de interrupçõe no PIC. Como este microcontrolador possui
//  apenas um único vetor de interrupção (0x0004), toda e qualquer interrupção será
//  tratada aqui. Se uma única interrupção for utilizada, o tratamento pode ser direto,
//  sem a necessidade de se testar qual int ocorreu. Porém se mais de uma estiver
//  em uso, a função deverá testar os flags das ints para saber qual ocorreu, extamente
//  como é feito em assembly.
//  O uso da diretriz "interrupt 0" é importante, pois é assim que o SDCC reconhece que
//  esta função é na verdade um serviço para tratamento das interrupções. Para PICs usar
//  apenas nível 0.
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*****************************************************************************
//static void isr(void) interrupt 0{
void __interrupt() pic_isr(void){

    INTCON &= ~0x80;                    //desabilita ints

    //analisa se é int do timer0
    //if((TMR1IF == 1) && (TMR1IE == 1)){ // é int do timer1
    //    trata_int_timer1();             // trata int
    //}
	
    //analisa se é int externa
    if((INTF == 1) && (INTE == 1)){     //é int externa
    	trata_int_ext();                //trata int
    }

    INTCON |= 0x80;                     // habilita novamente as ints

}

