//*****************************************************************************
// Arquivo pic_ds1302.c - módulo para uso do CI DS1302 (RTC))
// Desenvolvido por Márcio José Soares
//*****************************************************************************

//*****************************************************************************
// O Copyright deste programa está reservado para TESC Sistemas de Controle LTDA
// e seu conteúdo está protegido pela Lei de Direitos Autorais LEI Nº 9.610, de
// 19 de Fevereiro de 1998. É estritamente proibida a reprodução total ou
// parcial dos conteúdos aqui apresentados sem prévia autorização, por escrito,
// do autor.
//*****************************************************************************

//***********************************************************************
//* Definições para registradores e controle do DS1302
//***********************************************************************
#define sec_w 		0x80		//endereço do reg para conf. segundos - escrita
#define sec_r 		0x81		//endereço do reg para conf. segundos - leitura

#define min_w 		0x82		//endereço do reg para conf. minutos - escrita
#define min_r 		0x83		//endereço do reg para conf. minutos - leitura

#define hour_w      0x84		//endereço do reg para conf. horas - escrita
#define hour_r      0x85		//endereço do reg para conf. horas - leitura

#define date_w      0x86		// Microcontrolador: AVR MEGA//endereço do reg para conf. data - escrita
#define date_r      0x87		//endereço do reg para conf. data - leitura

#define month_w 	0x88		//endereço do reg para conf. mes - escrita
#define month_r 	0x89		//endereço do reg para conf. mes - leitura

#define day_w 		0x8A		//endereço do reg para conf. dia semana - escrita
#define day_r 		0x8B		//endereço do reg para conf. dia semana - leitura

#define year_w      0x8C		//endereço do reg para conf. ano - escrita
#define year_r      0x8D		//endereço do reg para conf. ano - leitura

#define w_protect   0x8E		//endereço do reg para proteção contra escrita
#define r_protect   0x8F		//endereço do reg para proteção contra escrita - leitura

#define t_rate_w 	0x90		//trickle rate escrita
#define t_rate_r 	0x91		//trickle rate leitura

#define ram_w		0xC0		//inicio do endereço da RAM presente no DS1302 - escrita
#define ram_r		0xC1		//inicio do endereço da RAM presente no DS1302 - leitura

//***********************************************************************
//Variáveis globais para o módulo DS1302.c
//***********************************************************************
//byte dia, mes, ano, dow;      		//variáveis para receber dados
byte hora, min, seg; 				//das funções do módulo DS1302.C

//***********************************************************************
// Função para converter binário->bcd 
// recebe 	- byte a converter 
// retorna 	- byte convertido
//***********************************************************************
byte bin_bcd(byte rec){

   byte lsb, msb, ret;//, i ,j;

	if(rec > 9){
  		msb = (rec / 10) & 0x0F;		//pega inteiro da divisão
		msb <<= 4;						//shift left x4
		lsb = (rec % 10) & 0x0F;		//resto da divisão
	}
	else{
		msb = 0;				//zero
		lsb = rec;
	}
	
	ret = msb | lsb;	//junta as partes

   return ret;			//retorna byte alterado
}


//***********************************************************************
// Função para resetar o DS1302
//
// após execução, DS1302 fica ligado!!!
//
// Entradas - nenhuma
// Saídas   - nenhuma
//***********************************************************************
void resetDS1302(void){
    
    //DS_RST = 1;             //levanta pino de reset
    DS_SCK = 0;             //abaixo pino de clock
    DS_RST = 0;             //abaixa pino de reset
    DS_RST = 1;             //levanta pino de reset   
    
}

//***********************************************************************
// Função para escrever um byte no DS1302
//
// Entradas - nenhuma
// Saídas   - nenhuma
//***********************************************************************
void writeDS1302(byte W_Byte){
	byte i;
	
	//DS_SCK = 1;     //levanta pinos
    //DS_DT = 1;
    //DS_RST = 1;

	for(i = 0; i < 8; i++){
		DS_DT = 0;                  // abaixa DT
		if(W_Byte & 0x01){
			DS_DT = 1;              // levanta DT
		}
		DS_SCK = 1;                 // levanta clock
		__delay_us(2);
		DS_SCK = 0;                 // abaixa clock
		__delay_us(2);
		W_Byte >>=1;
	}
}


//***********************************************************************
// Função para ler um byte do DS1302
//
// Entradas - nenhuma
// Saídas   - byte lido
//***********************************************************************
byte readDS1302(void){
    
	byte i;
	byte R_Byte, R_Byte2, TmpByte;

    //DS_RST = 1;
	DS_DT = 1 ;                     // levanta pino de dados

	R_Byte = 0x00;
	R_Byte2 = 0x00;
    DS_DT = 0;                      // abaixa pino de dados
    DS_DT = 0;                      // garantia e perde um ciclo
    
    TRISAbits.DS_DTtris = 1;        // reconfigura pino
                                    // pino RA7 agora é entrada
	for(i = 0; i < 4; i++){         // pega os 4 primeiros bits
		TmpByte = 0;
		if(DS_DT == 1)              // lê pinos de dados
			TmpByte = 1;            // faz temp igual 00000001
		TmpByte <<= 7;              // shift a esquerda 7x
		R_Byte >>= 1;               // shifth a direita 1x
		R_Byte |= TmpByte;          // junta as partes

		DS_SCK = 1;                 // levanta clock
		__delay_us(2);
		DS_SCK = 0;                 // abaixa clock
		__delay_us(2);
	}
	
	for(i = 0; i < 4; i++){         // pega os próximos 4 bits
		TmpByte = 0;
		if(DS_DT == 1)
			TmpByte = 1;
		TmpByte <<= 7;
		R_Byte2 >>= 1;
		R_Byte2 |= TmpByte;

		DS_SCK = 1;                 // levanta clock
		__delay_us(2);
		DS_SCK = 0;                 // abaixa clock
		__delay_us(2);
	}
	R_Byte >>= 4;                   //shift a direita 4x
	R_Byte2 >>= 4;              
	R_Byte = (R_Byte2 * 10) + R_Byte;
    
    TRISAbits.DS_DTtris = 0;       //RA7 volta a ser saída
    
	return R_Byte;
}

//***********************************************************************
// Função para ler um dado do DS1302
//
// Entradas - endereço do registrador a ser lido
// Saídas   - byte lido
//***********************************************************************
byte read_b_DS1302(byte w_byte){
	
	byte temp;

	resetDS1302();                  // reseta DS1302
	writeDS1302(w_byte);            // envia registro a ser lido
	temp = readDS1302();            // lê dado retornado
	DS_RST = 0;                     // abaixa reset
	DS_SCK=0;                       // abaixa clock
    
	return temp;
}

//***********************************************************************
// Função para escrever dados no DS1302
//
// Entradas - endereço do registrador e byte a ser escrito
// Saídas   - nenhuma
//***********************************************************************
void write_b_DS1302(byte w_byte, byte w_2_byte){
    
	resetDS1302();                  // resetaDS1302
	writeDS1302(w_byte);            // envia end do registro
	writeDS1302(w_2_byte);          // envia byte a ser escrito no reg
    DS_RST = 0;                     // abaixa reset
	DS_SCK=0;                       // abaixa clock
    
}

//***********************************************************************
// Função para inicializar o DS1302
//
// Entradas - nenhuma
// Saídas   - nenhuma
//
// valor a ser inserido no Ds1302 deve estar em BCD!
// fazer a conversão antes de usar write_byte!
//***********************************************************************
void DS1302_init(void){

    unsigned char x;
    
	write_b_DS1302(w_protect, 0x00);	//habilita escrita
	write_b_DS1302(t_rate_w, 0x00);		//desabilita carga

    x = read_b_DS1302(sec_r);
    if((x & 0x80)!=0x00){               //se bit 7 dos segundos estiver em
        write_b_DS1302(sec_w,0x00);     //necessário habilitar a contagem!
    }
}

//***********************************************************************
// Função para atualizar hora no DS1302
//
// Entradas - nenhuma
// Saídas   - nenhuma
//
// valor a ser inserido no Ds1302 deve estar em BCD!
// fazer a conversão antes de usar write_byte!
//***********************************************************************
void DS1302_set_time(void){
    
    hora = bin_bcd(hora);
	write_b_DS1302(hour_w, hora | tipohora);            //escreve hora
	write_b_DS1302(min_w, bin_bcd(min));                //escreve minutos
	write_b_DS1302(sec_w, bin_bcd(seg) & 0x7F);         //escreve segundos
}

//***********************************************************************
// Função para ler a hora inserida no DS1302
//
// Entradas - endereço do registrador e byte a ser escrito
// Saídas   - nenhuma
//
// valor recebido do DS1302 em binário... necessário sua conversão
// pela função que irá manipular o dado
//***********************************************************************
void DS1302_get_time(void){
 
    hora = read_b_DS1302(hour_r);       //pega hora
    if(tipohora)                       //se é padrão 12H limpa bits 7,6 e 5
        hora &= 0x1F;
    
    min = read_b_DS1302(min_r);         //pega minutos
	seg = (read_b_DS1302(sec_r) & 0x7F);//pega segundos, mas limpa bit7

}

