//******************************************************************************
// Projeto - RELOGIO BINARIO V2 (NERD)
// Desenvolvido por Márcio José Soares
//
// Arquivo - relogio_nerd.h - configurações, definições e variáveis globais
//******************************************************************************

//******************************************************************************
// Este arquivo é parte integrante do artigo Relógio Binário com
// microcontrolador. Maiores informações sobre o projeto podem ser obtidas
// no referido artigo publicado na Revista Eletronica Total
//******************************************************************************

//******************************************************************************
// O Copyright deste programa está reservado para Márcio José Soares e seu
// conteúdo está protegido pela Lei de Direitos Autorais LEI N° 9.610,
// de 19 de Fevereiro de 1998. É estritamente proibida o uso comercial como
// também a publicação total ou parcial dos conteúdos aqui apresentados
// sem prévia autorização, por escrito, do autor.
//******************************************************************************

#ifndef RELOGIO_BINARIO_H
#define	RELOGIO_BINARIO_H

//******************************************************************************
// Configurações para o microcontrolador PIC16F628
//******************************************************************************
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSC oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF      // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is digital input, MCLR internally tied to VDD)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

//*****************************************************************************
// Definições importantes do módulo
//*****************************************************************************
// Se nenhuma freqüência foi definida, assume 4MHz
// Esta definição é exigida para calibrar as funções __delay_us() e __delay_ms()
#ifndef _XTAL_FREQ
 #define _XTAL_FREQ 4000000
#endif

//******************************************************************************
// Arquivos incluídos no módulo
//******************************************************************************
#include <xc.h>

//*****************************************************************************
// Não há uma função C equivalente, então é preciso usar uma linha em ASM.
// Obs.: Interrupções tipo "pino" podem acordar o PIC, mas ints tipo "timer" não
#define sleep() asm("sleep")

// Nao Operador 
#define nop()   asm("nop")

#define byte uint8_t
#define TRUE 1
#define FALSE 0
#define tempo 3

//******************************************************************************
// Definições para o módulo - relogio_nerd.c
//******************************************************************************
#define  LINHAS   PORTA          //linhas em RA0 a RA3

#define  LIN1     RA0         //linha 1 na matriz
#define  LIN2     RA1         //linha 2 na matriz
#define  LIN3     RA2         //linha 3 na matriz
#define  LIN4     RA3         //linha 4 na matriz

#define  COLUNAS  PORTB          //colunas em RB2 a RB7

#define  COL1     RB2         //coluna 1 na matriz
#define  COL2     RB3         //coluna 2 na matriz
#define  COL3     RB4         //coluna 3 na matriz
#define  COL4     RB5         //coluna 4 na matriz
#define  COL5     RB6         //coluna 5 na matriz
#define  COL6     RB7         //coluna 6 na matriz

#define  AJT1     RA4         //avança tempo a cada 0,3 ms no modo ajuste
#define  AJT2     RA5         //troca de minutos para hora no modo ajuste
#define  F24H     RB1         //define formato 24 horas

#define  DS_SCK     RB5
#define  DS_DT      RA7       //DT no RA7
#define  DS_DTtris  TRISA7
#define  DS_RST     RA6

#define  HF24     0x17           //23
#define  HF12     0x0C           //12
#define  HI24     0x00           //00
#define  HI12     0x01           //01

//******************************************************************************
// Variáveis Globais do módulo - relogio_nerd.c
//******************************************************************************

unsigned char conta;             //variável para contar pulsos de int
unsigned char est_ajt;           //variável da máquina de estados do modo
                                 //ajuste
unsigned char prima;             //determina se ajustou alguma vez
unsigned char hora_final;        //formato da hora a ser mostrada (final)
unsigned char hora_inicial;      //formato da hora a ser mostrada (inicial)
unsigned char tipohora;

struct time{                    //estutura para relógio
   unsigned char horas;          //1 a 24
   unsigned char minutos;        //1 a 60
   unsigned char segundos;       //1 a 60
} horario;


//******************************************************************************
// Outros arquivos incluídos no módulo
//******************************************************************************
#include "pic_intext.c"         //módulo de uso da int externa
#include "my_ints.c"          	//módulo para tratamento das interrupções do PIC
#include "pic_ds1302.c"         //módulo de controle do DS1302
   
#ifdef	__cplusplus
extern "C" {
#endif

#ifdef	__cplusplus
}
#endif

#endif	/* RELOGIO_BINARIO_H */

